#!/usr/bin/env python
#-*- coding: UTF-8 -*-

import wx
from . import encfsmandir
from   .encfsmanmisc import *

class EditDialog(wx.Dialog):
    
    ID_START = wx.NewId()
    ID_BTN_UNCRYPTED = wx.NewId()
    ID_BTN_ENCRYPTED = wx.NewId()
    ID_CB_PASS = wx.NewId()
    ID_CB_AUTO = wx.NewId()
    
    def __init__(self, parent, writeToLogwin, config, mapping = None, edit = True):        
        """   """
        wx.Dialog.__init__(self, parent, -1, TITLE + _(' - Add / Edit mapping'))
        self.Bind(wx.EVT_CLOSE, self.onOk)
        self.parent = parent
        self.mapping = mapping
        self.writeToLogwin = writeToLogwin
        self.config = config
        btnadj = int(parent.config['Gui']['norm'] == 'False')
        background_color = wx.SystemSettings_GetColour(READONLY_COLOR)
        
        sizer = wx.BoxSizer(wx.VERTICAL)
        sb = wx.StaticBox(self, -1, _(' Define values for mapping '))
        row1 = wx.StaticBoxSizer(sb, wx.VERTICAL)
        fgsz = wx.FlexGridSizer(7, 3, 0, 0)
        fgsz.AddGrowableCol(1)
        st = wx.StaticText(self, -1, _('Unencrypted dir:'))
        fgsz.Add(st, 0, wx.ALL, GAP)
        self.ctrluc = wx.TextCtrl(self, -1, self.mapping.unencdir, style = wx.TE_READONLY)
        self.ctrluc.SetBackgroundColour(background_color)
        fgsz.Add(self.ctrluc, 0, wx.ALL | wx.GROW, GAP)
        self.btnuc = wx.Button(self, self.ID_BTN_UNCRYPTED, _('Browse')) 
        self.Bind(wx.EVT_BUTTON, self.onUnencrypted, id = self.ID_BTN_UNCRYPTED)
        fgsz.Add(self.btnuc, 0, wx.ALL, GAP)
        
        st = wx.StaticText(self, -1, _('Encrypted dir:'))
        fgsz.Add(st, 0, wx.ALL, GAP)
        self.ctrlec = wx.TextCtrl(self, -1, self.mapping.encdir, style = wx.TE_READONLY)
        self.ctrlec.SetBackgroundColour(background_color)
        fgsz.Add(self.ctrlec, 0, wx.ALL | wx.GROW, GAP)
        self.btnec = wx.Button(self, self.ID_BTN_ENCRYPTED, _('Browse')) 
        self.Bind(wx.EVT_BUTTON, self.onEncrypted, id = self.ID_BTN_ENCRYPTED)
        fgsz.Add(self.btnec, 0, wx.ALL, GAP)

        st = wx.StaticText(self, -1, _('Autocreate:'))
        fgsz.Add(st, 0, wx.ALL, GAP)
        self.cbc = wx.CheckBox(self, self.ID_CB_AUTO, _(' (if checked - create EncFS when needed)    '))
        fgsz.Add(self.cbc, 0, wx.ALL, GAP)
        self.cbc.SetValue(mapping.autocreate == 'True')
        self.Bind(wx.EVT_CHECKBOX, self.onAutocreate, id = self.ID_CB_AUTO)       
        row1.Add(fgsz, 0, wx.EXPAND | wx.ALL, GAP)
        fgsz.AddStretchSpacer()

        st = wx.StaticText(self, -1, _('Automount: '))
        fgsz.Add(st, 0, wx.ALL, GAP)
        mountlist = ['False', 'True']
        self.co_mount = wx.ComboBox(self, -1, self.mapping.automount, 
                                     choices = mountlist, style = wx.CB_DROPDOWN | wx.CB_READONLY)       
        fgsz.Add(self.co_mount, 0, wx.ALL | wx.GROW, GAP)
        fgsz.AddStretchSpacer()
        self.Bind(wx.EVT_COMBOBOX, self.onMountCombo)        
            
        st = wx.StaticText(self, -1, _('Passphrase:'))
        fgsz.Add(st, 0, wx.ALL, GAP)
        self.tricksizer = wx.BoxSizer(wx.VERTICAL)
        self.ctrlppp = wx.TextCtrl(self, -1, self.mapping.passphrase, style = wx.TE_PASSWORD)
        self.tricksizer.Add(self.ctrlppp, 0, wx.GROW)
        self.ctrlppn = wx.TextCtrl(self, -1, self.mapping.passphrase)
        self.Bind(wx.EVT_TEXT, self.onEnableOkButton, self.ctrlppn)
        self.Bind(wx.EVT_TEXT, self.onEnableOkButton, self.ctrlppp)
        self.ctrlppn.Hide()
        self.tricksizer.Add(self.ctrlppn, 0, wx.GROW)        
        fgsz.Add(self.tricksizer, 0, wx.ALL | wx.GROW, GAP)
        fgsz.AddStretchSpacer()
        st = wx.StaticText(self, -1, _('Show passphrase:'))
        fgsz.Add(st, 0, wx.ALL, GAP)
        self.cbp = wx.CheckBox(self, self.ID_CB_PASS, _(' (if checked - unhide passphrase)    '))
        fgsz.Add(self.cbp, 0, wx.ALL, GAP)
        self.Bind(wx.EVT_CHECKBOX, self.onPassphrase, id = self.ID_CB_PASS)
        fgsz.AddStretchSpacer()
                
        self.butok = wx.Button(self, wx.ID_OK)
        self.butok.Enable(edit)
        self.Bind(wx.EVT_BUTTON, self.onOk, id = wx.ID_OK)
        butcancel = wx.Button(self, wx.ID_CANCEL)
        row2 = wx.BoxSizer(wx.HORIZONTAL)
        if not btnadj: row2.AddStretchSpacer()
        row2.Add(self.butok, btnadj, wx.GROW | wx.RIGHT, GAP)
        row2.Add(butcancel, btnadj, wx.GROW | wx.LEFT, GAP)

        sizer.Add(row1, 0, wx.GROW | LRT, BORDER)
        sizer.Add(row2, 0, wx.GROW | wx.ALL, BORDER)
        self.SetSizer(sizer)
        self.Fit()
        self.onEnableOkButton()
    #-------------------------------------------------------------------------------------------

    def onEnableOkButton(self, event = None):
        if self.ctrlppn.IsShown():
            self.mapping.passphrase = self.ctrlppn.GetValue()
        else:
            self.mapping.passphrase = self.ctrlppp.GetValue()

        if len(self.mapping.unencdir) and len(self.mapping.encdir) and len(self.mapping.passphrase):
            self.butok.Enable(True)
        else:
            self.butok.Enable(False)
    #-------------------------------------------------------------------------------------------               

    def onUnencrypted(self, event):
        dlg = encfsmandir.DirDialog(self, TITLE + ' - select unencrypted dir', self.config,
                                    self.mapping.unencdir, mode = 'opend')
        if dlg.ShowModal() == wx.ID_OPEN:
            path = os.path.normpath(dlg.GetPath())
            self.mapping.unencdir = path
            self.ctrluc.SetValue(path)
        self.onEnableOkButton()
        dlg.Destroy()
    #-------------------------------------------------------------------------------------------        

    def onEncrypted(self, event):
        dlg = encfsmandir.DirDialog(self, TITLE + ' - select encrypted dir', self.config,
                                   self.mapping.encdir, mode = 'opend')
        if dlg.ShowModal() == wx.ID_OPEN:
            path = os.path.normpath(dlg.GetPath())
            self.mapping.encdir = path
            self.ctrlec.SetValue(path)
        self.onEnableOkButton()
        dlg.Destroy()
    #-------------------------------------------------------------------------------------------        

    def onMountCombo(self, event):
        self.mapping.automount = self.co_mount.GetValue()
    #-------------------------------------------------------------------------------------------        

    def onPassphrase(self, event):
        if self.cbp.GetValue():
            self.ctrlppn.SetValue(self.ctrlppp.GetValue())
            self.ctrlppp.Hide()
            self.ctrlppn.Show()
            self.ctrlppn.SetFocus()
            self.ctrlppn.SetInsertionPointEnd()
        else:
            self.ctrlppp.SetValue(self.ctrlppn.GetValue())
            self.ctrlppn.Hide()
            self.ctrlppp.Show()
            self.ctrlppp.SetFocus()
            self.ctrlppp.SetInsertionPointEnd()
        self.tricksizer.Layout()
        self.Refresh()            
    #-------------------------------------------------------------------------------------------

    def onAutocreate(self, event):
        self.mapping.autocreate = str(self.cbc.GetValue())
    #-------------------------------------------------------------------------------------------
    
    def onOk(self, event):
        event.Skip()
# - class EditDialog(wx.Dialog):----------------------------------------------------------------

