#!/usr/bin/env python
#-*- coding: UTF-8 -*-

import sys
import subprocess
import os
import os.path
import codecs
import shutil
import threading
import wx
import wx.html
import __builtin__
__builtin__.__dict__['_'] = wx.GetTranslation
import logging
import logging.handlers
import webbrowser
from   time import strftime
from   .encfsmanmisc import *

try:
    from agw import aui
except ImportError: # if it's not there locally, try the wxPython lib.
    import wx.lib.agw.aui as aui

try:
    import wx.lib.evtmgr as evtmgr
    import wx.lib.intctrl
except:    
    class PanicApp(wx.App):
        def OnInit(self):
            print('\n< < <  P A N I C  > > >\n')
            print("Can not start EncFSman because the lib 'intctrl' is missing!")
            print('Check your installation. The system reports, that you are')
            print(WXVERSION, '- but it has to be 2.8 or better!')
            print()
            print('Please install wxPython 2.8 (package python-wxgtk2.8 on')
            print('Ubuntu / Kubuntu).')
            return True
            
    app = PanicApp(redirect = True)
    app.MainLoop()
   
from . import encfsmanabout
from . import encfsmansystray
from . import encfsmancustbar
from . import encfsmanupdate
from . import encfsmanprefs
from . import encfsmaninstall
from . import encfsmanedit
from . import encfsmanpopup
from . import encfsmanpasswd

from   .configobj import ConfigObj
from   .singleinstance import SingleInstance
try:
    import keyring
    keyringisthere = True
except:
    keyringisthere = False

try:
    import notify2 as pynotify
    pynotify.init(TITLE)
    notifyisthere = True
except:
    try:
        import pynotify
        pynotify.init(TITLE)
        notifyisthere = True
    except:
        notifyisthere = False    

class Mapping(object):
    def __init__(self, unencdir = '', encdir = '', automount = 'False',
                       passphrase = '', idstring = '', autocreate = 'True',
                       inittuple = None):
        if inittuple == None:
            self.unencdir = unencdir
            self.encdir = encdir
            self.automount = automount
            self.passphrase = passphrase
            self.idstring = idstring
            self.autocreate = autocreate
        else:
            items = eval(inittuple)
            self.unencdir = items[0]
            self.encdir = items[1]
            self.automount = items[2]
            self.passphrase = items[3]
            self.idstring = items[4]
            self.autocreate = items[5]            
    #-------------------------------------------------------------------------------------------
    
    def getTuple(self):
        return (self.unencdir, self.encdir, self.automount, '', self.idstring, self.autocreate)
# - class Mapping(object):----------------------------------------------------------------------

           
class EncFSmanFrame(wx.Frame):   
    ID_LOGWIN = wx.NewId()
    ID_SYSTRAY = wx.NewId()
    ID_EXIT = wx.NewId()
    ID_CREATE = wx.NewId()
    ID_MOUNT = wx.NewId()
    ID_UNMOUNT = wx.NewId()
    ID_ADDMAPPING = wx.NewId()
    ID_DELMAPPING = wx.NewId()
    ID_EDITMAPPING = wx.NewId()
    ID_WEBSITE = wx.NewId()
    ID_UPDATE = wx.NewId()
    ID_INSTALL = wx.NewId()
    ID_ABOUT = wx.NewId()
    ID_HELP = wx.NewId()
    ID_KEYRING = wx.NewId()
    ID_PREFS = wx.NewId()
    ID_LISTCTRL = wx.NewId()
    ID_INI_PERSP = wx.NewId()
    ID_PROPER = wx.NewId()
    

    def __init__(self, parent, id, title, instance, keyringisthere):
        self.instance = instance        # this object should live until termination
        self.keyringisthere = keyringisthere
        self.handleConfigData('r')
        
        langindex = self.config['Gui']['language'].index(self.config['Gui']['curlang'])
        wxlanguages = (wx.LANGUAGE_DEFAULT, wx.LANGUAGE_ENGLISH, wx.LANGUAGE_GERMAN)
        self.mylocale = wx.Locale(wxlanguages[langindex])
        self.mylocale.AddCatalogLookupPathPrefix(LOCALEDIR)
        self.mylocale.AddCatalog(TITLE)        
        
        windowpos = (wx.Point(int(self.config['Window']['posx']),
                               int(self.config['Window']['posy'])))
        windowsize = (wx.Size(int(self.config['Window']['sizex']),
                              int(self.config['Window']['sizey'])))
        wx.Frame.__init__(self, parent, id, title, pos = windowpos, size = windowsize)  

        inifilebak = file(self.inifilename.replace('.ini', '.ini.bak'), 'w')
        self.config.write(inifilebak)
        inifilebak.close()
        if self.config['Gui']['systray'] == 'True':
            self.Bind(wx.EVT_CLOSE, self.onSysTray)
        else:
            self.Bind(wx.EVT_CLOSE, self.onCloseWindow)

        wx.InitAllImageHandlers()
        iconfile = ICONPATH + os.sep + APPICON
        if os.path.exists(iconfile):
            image = wx.Image(iconfile, wx.BITMAP_TYPE_PNG)
            image.Rescale(SCALE, SCALE, wx.IMAGE_QUALITY_NORMAL)
            self.icon = wx.IconFromBitmap(image.ConvertToBitmap())
            self.SetIcon(self.icon)
        else:
            self.icon = None

        logfilename = self.inifilename.replace('.ini', '-Action.log')
        logpath = os.path.join(HOMEDIR, logfilename)
        self.logger = logging.getLogger('ActionLog')
        self.logger.setLevel(logging.INFO)
        handler = logging.handlers.RotatingFileHandler(logpath, 'a', ACTION_LOG_SIZE, 4)
        formatter = logging.Formatter("%(asctime)s -- %(message)s", "%Y-%m-%d %H:%M:%S")
        handler.setFormatter(formatter)        
        self.logger.addHandler(handler)
        
        self.initMenu()
        self.sb = encfsmancustbar.CustomStatusBar(self)
        self.SetStatusBar(self.sb)        

        collist = self.config['Gui']['hlcolor']
        self.hlcolor = (int(collist[0]), int(collist[1]), int(collist[2]))
                
        self._mgr = aui.AuiManager()
        self._mgr.SetManagedWindow(self)
                
        self.logwin = wx.TextCtrl(self, -1, size = wx.Size(-1, 80), style = wx.TE_RICH | wx.TE_MULTILINE | wx.TE_DONTWRAP | wx.TE_READONLY)
        style = wx.LC_REPORT
        if self.config['Gui']['grid'] == 'True':
            style |= wx.LC_HRULES | wx.LC_VRULES
        self.listwin = wx.ListCtrl(self, self.ID_LISTCTRL, style = style)
        imagelist = wx.ImageList(16, 16)
        imagelist.Add(wx.ArtProvider.GetBitmap(wx.ART_NORMAL_FILE, wx.ART_OTHER, wx.Size(16, 16)))
        self.listwin.AssignImageList(imagelist, wx.IMAGE_LIST_SMALL)        
        
        self.listwin.InsertColumn(0, _('Unencrypted dir'))
        self.listwin.InsertColumn(1, _('Encrypted dir'))
        self.listwin.InsertColumn(2, _('Automount'))
        self.listwin.InsertColumn(3, _('Mounted')) 
        self.listwin.SetColumnWidth(0, int(self.config['Window']['lwcol0']))
        self.listwin.SetColumnWidth(1, int(self.config['Window']['lwcol1']))
        self.listwin.SetColumnWidth(2, int(self.config['Window']['lwcol2']))
        self.listwin.SetColumnWidth(3, int(self.config['Window']['lwcol3']))

        self.myfont = wx.Font(int(self.config['Gui']['fsize']), wx.DEFAULT, wx.NORMAL, wx.NORMAL, False, self.config['Gui']['font'])
        self.listwin.SetFont(self.myfont)
        self.Bind(wx.EVT_LIST_ITEM_ACTIVATED, self.onItemDoubleClick, self.listwin)
        self.Bind(wx.EVT_LIST_COL_END_DRAG, self.onColSize, self.listwin)
        self._mgr.AddPane(self.listwin, aui.AuiPaneInfo().Name('listwin').Center().CloseButton(False).MaximizeButton(True))
        evtmgr.eventManager.Register(self.onContext, wx.EVT_RIGHT_DOWN, self.listwin)
        self._mgr.AddPane(self.logwin, aui.AuiPaneInfo().Name('logwin').Bottom())
        gradient = aui.AUI_GRADIENT_HORIZONTAL
        self._mgr.GetArtProvider().SetMetric(aui.AUI_DOCKART_GRADIENT_TYPE, gradient)
        flags = self._mgr.GetAGWFlags()
        flags &= ~aui.AUI_MGR_TRANSPARENT_HINT
        flags &= ~aui.AUI_MGR_VENETIAN_BLINDS_HINT
        self._mgr.SetAGWFlags(flags)       
        flag = aui.AUI_MGR_RECTANGLE_HINT
        self._mgr.SetAGWFlags(self._mgr.GetAGWFlags() ^ flag)
        self._mgr.Update()
        self.Bind(aui.EVT_AUI_PANE_CLOSE, self.onPaneClose)
        defaultperspective = self._mgr.SavePerspective()
        if len(self.config['Window']['defaultperspective']) == 0:
            self.config['Window']['defaultperspective'] = defaultperspective       
        if len(self.config['Window']['perspective']):
            self._mgr.LoadPerspective(self.config['Window']['perspective'])
        
        self._mgr.GetPane('listwin').Caption(_('EncFS mapping list'))
        self._mgr.GetPane('logwin').Caption(_('Log messages'))
        self._mgr.Update()

        self.tbicon = encfsmansystray.SysTrayIcon(self, self.icon, TITLE)          
        self.initContextMenu()

        self.mainmenu.Enable(self.ID_INSTALL, False)

        encfsfound, encfspath = searchFile(self.config['Misc']['encfsbin'])
        if encfsfound:
            self.config['Misc']['encfsbin'] = encfspath
        encfscfound, encfscpath = searchFile(self.config['Misc']['encfsctl'])
        if encfscfound:
            self.config['Misc']['encfsctl'] = encfscpath
        fusefound, fusepath = searchFile(self.config['Misc']['fusebin'])
        if fusefound:
            self.config['Misc']['fusebin'] = fusepath
         
        # first try to use the new SecretServiceKeyring
        try:
            keyring.set_keyring(keyring.backends.SecretService.Keyring())   # Kubuntu 14.04
        except:
            keyring.set_keyring(keyring.backend.SecretServiceKeyring())     # < Kubuntu 14.04   
        try:
            keyring.set_password(TITLE, USER, TITLE)        
        except:     # if thats not possible, use the system default
            keyring.core.init_backend()        
        try:
            kr =  keyring.get_keyring()
            krsplit = str(kr).split('.')
            self.krbackend = krsplit[2]
        except:
            self.krbackend = '???'
                
        self.checkKeyring()
        self.updateListwin()
             
        if self.config['Misc']['update'] == 'True':
            self.updatecheck = True
            self.updatetimer = wx.Timer(self, -1) 
            self.Bind(wx.EVT_TIMER, self.onUpdate, self.updatetimer) 
            self.updatetimer.Start(8000, oneShot = True)
        else:
            self.updatecheck = False            
        self.writeToLogwin(_('%s started ...') % (VERSION))
        
        self.retrycount = int(self.config['Misc']['retry'])
        self.automounttimer = wx.Timer(self, -1)
        self.Bind(wx.EVT_TIMER, self.onAutomount, self.automounttimer)
        self.automounttimer.Start(4000, oneShot = True)
        
        self.minimizetimer = wx.Timer(self, -1)
        self.Bind(wx.EVT_TIMER, self.onSysTray, self.minimizetimer)
        if self.config['Gui']['startmin'] == 'True':
            self.minimizetimer.Start(10, oneShot = True)        
    #-------------------------------------------------------------------------------------------

    def initMenu(self):
        self.mainmenu = wx.MenuBar()         
        # File menu
        menu = wx.Menu()                     
        menu.AppendSeparator() 
        item = wx.MenuItem(menu, self.ID_EXIT, _('E&xit EncFSman\tCTRL-X'), _('exit the application'))
        item.SetBitmap(loadIcon('exit.png'))                 
        menu.AppendItem(item)
        self.Bind(wx.EVT_MENU, self.onCloseWindow, id = self.ID_EXIT)
        self.mainmenu.Append(menu, _('&File'))
        
        # Edit menu
        menu = wx.Menu()
        item = wx.MenuItem(menu, self.ID_ADDMAPPING, _('&Add mapping\tCTRL-A'), _('add new mapping'))
        item.SetBitmap(loadIcon('addmapping.png'))
        menu.AppendItem(item)
        self.Bind(wx.EVT_MENU, self.onAddMapping, id = self.ID_ADDMAPPING)
        menu.AppendSeparator()
        item = wx.MenuItem(menu, self.ID_DELMAPPING, _('&Delete mapping\tCTRL-D'), _('delete mapping'))
        item.SetBitmap(loadIcon('delmapping.png'))
        menu.AppendItem(item)
        self.Bind(wx.EVT_MENU, self.onDelMapping, id = self.ID_DELMAPPING)
        item = wx.MenuItem(menu, self.ID_EDITMAPPING, _('&Edit mapping\tCTRL-E'), _('edit mapping components'))
        item.SetBitmap(loadIcon('editmapping.png'))
        self.Bind(wx.EVT_MENU, self.onEditMapping, id = self.ID_EDITMAPPING)
        menu.AppendItem(item)        
        self.mainmenu.Append(menu, _('&Edit'))

        # View menu
        menu = wx.Menu()
        menu.Append(self.ID_LOGWIN, _('&Log messages\tSHIFT-ALT-L'), _('show log window'), wx.ITEM_CHECK)
        self.Bind(wx.EVT_MENU, self.onLogWin, id = self.ID_LOGWIN)
        menu.AppendSeparator()
        menu.Append(self.ID_INI_PERSP, _('&Default perspective'), _('set views to default perspective'))
        self.Bind(wx.EVT_MENU, self.onPerspective, id = self.ID_INI_PERSP)        
        self.mainmenu.Append(menu, _('&View'))
        
        # Options menu
        menu = wx.Menu()
        item = wx.MenuItem(menu, self.ID_CREATE, _('Crea&te and mount\tCTRL-T'), _('create EncFS and mount'))
        item.SetBitmap(loadIcon('mount.png'))
        menu.AppendItem(item)
        self.Bind(wx.EVT_MENU, self.onCreate, id = self.ID_CREATE)        
        menu.AppendSeparator()
        item = wx.MenuItem(menu, self.ID_MOUNT, _('&Mount\tCTRL-M'), _('mount mapping'))
        item.SetBitmap(loadIcon('mount.png'))
        menu.AppendItem(item)
        self.Bind(wx.EVT_MENU, self.onMount, id = self.ID_MOUNT)        
        item = wx.MenuItem(menu, self.ID_UNMOUNT, _('&Unmount\tCTRL-U'), _('unmount mapping'))
        item.SetBitmap(loadIcon('unmount.png'))
        menu.AppendItem(item)
        self.Bind(wx.EVT_MENU, self.onUnmount, id = self.ID_UNMOUNT)        
        menu.AppendSeparator()
        item = wx.MenuItem(menu, self.ID_PREFS, _('Pre&ferences\tCTRL-F'), _('set preferences'))
        item.SetBitmap(loadIcon('prefs.png'))
        menu.AppendItem(item)
        self.Bind(wx.EVT_MENU, self.onPreferences, id = self.ID_PREFS)        
        menu.AppendSeparator()
        item = wx.MenuItem(menu, self.ID_SYSTRAY, _('S&ystray icon\tCTRL-Y'), _('minimize app to systray'))
        item.SetBitmap(loadIcon('systray.png'))
        menu.AppendItem(item)
        self.Bind(wx.EVT_MENU, self.onSysTray, id = self.ID_SYSTRAY)
        self.mainmenu.Append(menu, _('&Options'))
        
        # Help menu
        menu = wx.Menu()
        item = wx.MenuItem(menu, self.ID_HELP, _('&Help\tF1'), _('html help docs'))
        item.SetBitmap(loadIcon('help.png'))  
        menu.AppendItem(item)
        self.Bind(wx.EVT_MENU, self.onHandbook, id = self.ID_HELP)
        menu.AppendSeparator()
        item = wx.MenuItem(menu, self.ID_WEBSITE, _('EncFSman &Website\tCTRL-W'), _('EncFSman Website'))
        item.SetBitmap(loadIcon('website.png'))  
        menu.AppendItem(item)
        self.Bind(wx.EVT_MENU, self.onWebsite, id = self.ID_WEBSITE)
        menu.AppendSeparator()     
        item = wx.MenuItem(menu, self.ID_UPDATE, _('EncFSman U&pdates\tCTRL-P'), _('Download newest version'))
        item.SetBitmap(loadIcon('update.png'))  
        menu.AppendItem(item)
        self.Bind(wx.EVT_MENU, self.onUpdate, id = self.ID_UPDATE)                
        item = wx.MenuItem(menu, self.ID_INSTALL, _('install EncFSman Updates'), _('install newest version'))
        item.SetBitmap(loadIcon('update.png'))  
        menu.AppendItem(item)
        self.Bind(wx.EVT_MENU, self.onInstallUpdate, id = self.ID_INSTALL)                
        menu.AppendSeparator()        
        item = wx.MenuItem(menu, self.ID_ABOUT, _('About EncFSman\tSHIFT-CTRL-A'), _('about the application'))
        item.SetBitmap(loadIcon('about.png'))  
        menu.AppendItem(item)
        self.Bind(wx.EVT_MENU, self.onAbout, id = self.ID_ABOUT)
        self.mainmenu.Append(menu, _('&Help'))

        self.SetMenuBar(self.mainmenu)
        
        self.mainmenu.Check(self.ID_LOGWIN, self.config['View']['logwin'] == 'True')
    #-------------------------------------------------------------------------------------------
        
    def initContextMenu(self):
        self.popupID0 = wx.NewId()  
        self.popupID1 = wx.NewId()          
        self.popupID2 = wx.NewId()
        self.popupID3 = wx.NewId()
        self.popupID4 = wx.NewId()
        self.popupID5 = wx.NewId()
        self.popupID6 = wx.NewId()  
        self.Bind(wx.EVT_MENU, self.onMount, id = self.popupID0)
        self.Bind(wx.EVT_MENU, self.onUnmount, id = self.popupID1)
        self.Bind(wx.EVT_MENU, self.onAddMapping, id = self.popupID2)
        self.Bind(wx.EVT_MENU, self.onDelMapping, id = self.popupID3)
        self.Bind(wx.EVT_MENU, self.onEditMapping, id = self.popupID4)
        self.Bind(wx.EVT_MENU, self.onPreferences, id = self.popupID5)
        self.Bind(wx.EVT_MENU, self.onSysTray, id = self.popupID6)
    #-------------------------------------------------------------------------------------------
    
    def onContext(self, event):
        """ context menu handler """
        menu = wx.Menu()
        item = wx.MenuItem(menu, self.popupID0, _('&Mount\tCTRL-M'))
        item.SetBitmap(loadIcon('mount.png'))
        menu.AppendItem(item)
        item = wx.MenuItem(menu, self.popupID1, _('&Unmount\tCTRL-U'))
        item.SetBitmap(loadIcon('unmount.png'))
        menu.AppendItem(item)
        menu.AppendSeparator()  
        item = wx.MenuItem(menu, self.popupID2, _('&Add mapping\tCTRL-A'))
        item.SetBitmap(loadIcon('addmapping.png'))
        menu.AppendItem(item)        
        item = wx.MenuItem(menu, self.popupID3, _('&Delete mapping\tCTRL-D'))
        item.SetBitmap(loadIcon('delmapping.png'))
        menu.AppendItem(item) 
        item = wx.MenuItem(menu, self.popupID4, _('&Edit mapping\tCTRL-E'))
        item.SetBitmap(loadIcon('editmapping.png'))
        menu.AppendItem(item)
        menu.AppendSeparator()
        item = wx.MenuItem(menu, self.popupID5, _('Pre&ferences\tCTRL-F'))
        item.SetBitmap(loadIcon('prefs.png'))
        menu.AppendItem(item)
        item = wx.MenuItem(menu, self.popupID6, _('S&ystray icon\tCTRL-Y'))
        item.SetBitmap(loadIcon('systray.png'))
        menu.AppendItem(item)        
        self.PopupMenu(menu)
        menu.Destroy()
    #-------------------------------------------------------------------------------------------      

    def onColSize(self, event):
        for i in range(self.listwin.GetColumnCount()):
            self.config['Window']['lwcol%i' % i] = self.listwin.GetColumnWidth(i)
    #-------------------------------------------------------------------------------------------
        
    def onPaneClose(self, event):
        mb = self.GetMenuBar()
        name = event.GetPane().name
        if name == 'logwin':
            submenuitem = mb.FindItemById(self.ID_LOGWIN)
            self.config['View']['logwin'] = 'False'
        submenuitem.Check(False)    
    #-------------------------------------------------------------------------------------------

    def onLogWin(self, event):
        mb = self.GetMenuBar()
        submenuitem = mb.FindItemById(self.ID_LOGWIN)
        if submenuitem.IsChecked():
            self._mgr.GetPane('logwin').Show()
        else:
            self._mgr.GetPane('logwin').Hide()
        self.config['View']['logwin'] = str(submenuitem.IsChecked())
        self._mgr.Update()
    #-------------------------------------------------------------------------------------------

    def onPerspective(self, event):
        self.setWindowDefaults()
        self.SetDimensions(int(self.config['Window']['posx']),
                           int(self.config['Window']['posy']),
                           int(self.config['Window']['sizex']),
                           int(self.config['Window']['sizey']))
        self.Refresh()
        self.config['View']['logwin'] = 'True'
        self.mainmenu.Check(self.ID_LOGWIN, True)
        self._mgr.LoadPerspective(self.config['Window']['defaultperspective'])
    #-------------------------------------------------------------------------------------------

    def writeToLogwin(self, info, color = None):
        try:
            start = self.logwin.GetInsertionPoint()
            self.logwin.WriteText('%s\n' % (info))
            stop = self.logwin.GetInsertionPoint() - 1
            if color != None:
                self.logwin.SetStyle(start, stop, wx.TextAttr(color))
            self.logwin.ShowPosition(-1)
            if self.config['Misc']['actionlog'] == 'True':
                self.logger.info('%s' % (info))
        except:
            pass
    #-------------------------------------------------------------------------------------------

    def taskbarNotify(self, message):
        if notifyisthere and self.config['Gui']['notify'] == 'True' and not self.tbicon.IsIconInstalled():
            imguri = ICONPATH + os.sep + 'musicman.png'
            notification = pynotify.Notification('%s - Version %s' % (TITLE, VERNUM), message, imguri)
            notification.set_urgency(pynotify.URGENCY_NORMAL)
            notification.set_timeout(5000) 
            notification.show()
    #-------------------------------------------------------------------------------------------

    def onColSize(self, event):
        for i in range(self.listwin.GetColumnCount()):
            self.config['Window']['lwcol%i' % i] = self.listwin.GetColumnWidth(i)
    #-------------------------------------------------------------------------------------------

    def checkKeyring(self, fetch = True):
        self.keyringisopen = False
        try:
            self.sb.setLed(RED)
        except:
            return        

        if self.keyringisthere:
            try:
                keyring.set_password(TITLE, USER, TITLE)        # EncFSman Id in keyring?
                if keyring.get_password(TITLE, USER) == TITLE:
                    self.keyringisopen = True
                    self.sb.setLed(GREEN)
                    if fetch:
                        for mapping in self.mappings:       # fetch all passphrases
                            mapping.passphrase = keyring.get_password(mapping.idstring, USER)
                            if mapping.passphrase == None: mapping.passphrase = ''
            except:
                pass
    #-------------------------------------------------------------------------------------------

    def getMountInfos(self):
        mtab = []
        try:
            lines = [line.strip('\n').split() for line in open('/proc/mounts', 'r').readlines()]
            for line in lines:
                if line[0] == 'encfs': mtab.append(line[1])
        except:
            pass
        return mtab
    #-------------------------------------------------------------------------------------------

    def updateListwin(self):
        dirstxt = ''
        mountcount = 0
        mtab = self.getMountInfos()
        self.listwin.DeleteAllItems()
        for index, mapping in enumerate(self.mappings):            
            item = self.listwin.InsertImageStringItem(index, mapping.unencdir, 0)
            self.listwin.SetStringItem(index, 1, mapping.encdir)
            self.listwin.SetStringItem(index, 2, mapping.automount)
            if mapping.unencdir in mtab:
                self.listwin.SetItemTextColour(item, self.hlcolor)
                self.listwin.SetStringItem(index, 3, 'True')
                mountcount += 1
                dirstxt += '\n%s' % mapping.unencdir
            else:
                self.listwin.SetStringItem(index, 3, 'False')
        infotxt = _('%s\ncurrently mounted (%d):%s') % (VERSION, mountcount, dirstxt)
        self.tbicon.setTitle(infotxt) 
    #-------------------------------------------------------------------------------------------

    def checkEncFSinitialized(self, mapping):
        if os.path.isfile(self.config['Misc']['encfsctl']):
            cmdline = '"%s" "%s"' % (self.config['Misc']['encfsctl'], mapping.encdir)
            proc = subprocess.Popen(cmdline, shell = True, stdin = subprocess.PIPE, stdout = subprocess.PIPE, stderr = subprocess.PIPE)
            stdoutdata, stderrdata = proc.communicate()
            if stderrdata == '' and 'EncFS' in stdoutdata:
                return True
        return False
    #-------------------------------------------------------------------------------------------

    def onCreate(self, event = None, mapping = None):
        if event != None:
            index, mapping = self.getSelectedMapping()
        
        self.createAndMount(mapping, True, True)
        self.updateListwin()
    #-------------------------------------------------------------------------------------------
    
    def createAndMount(self, mapping, withdialog = False, forcecreate = False):
        encfsbinarypath = self.config['Misc']['encfsbin']
        if not os.path.isfile(encfsbinarypath): return False
        
        if self.checkEncFSinitialized(mapping):     # mount only, because EncFS is created
            if withdialog:
                infotext = TITLE + _(' - Mount mapping')
                infotext += _(':\n\nwork in progress ...')
                infotext += _('\nPlease be patient - this operation can take some time!')
                popupwin = encfsmanpopup.PopupWin(self, infotext)
                pos = self.ClientToScreen((0,0))  # pos is the left top edge of the win
                sz = self.GetSize()               # sz is the size of the win
                popupwin.Position((pos[0] + sz[0] / 2 - 150, pos[1] - 50 - sz[1] / 2), (0, sz[1]))
                popupwin.Popup()                
                wx.SafeYield(self)
            cmdline = '"%s" -S "%s" "%s"' % (encfsbinarypath, mapping.encdir, mapping.unencdir)
            proc = subprocess.Popen(cmdline, shell = True, stdin = subprocess.PIPE, stdout = subprocess.PIPE, stderr = subprocess.PIPE)
            wx.SafeYield(self)
            stdoutdata, stderrdata = proc.communicate(input = mapping.passphrase)
        else:                                       # create and mount
            if mapping.autocreate == 'False' and not forcecreate: return False
            if not os.access(mapping.encdir, os.W_OK):  # can we write to the encrypted dir?
                if withdialog:
                    dlg = wx.MessageDialog(self, _("Encrypted dir '%s' is not writable.\n No encrypted filesystem created.") % (mapping.encdir) , TITLE + ' - Create encfs problem', wx.OK | wx.ICON_WARNING)
                    dlg.ShowModal()
                self.writeToLogwin(_("Create encrypted fs - '%s' is not created (not writable)") % (mapping.encdir))    
                return False
            if withdialog:
                infotext = TITLE + _(' - Create and mount mapping')
                infotext += _(':\n\nwork in progress ...')
                infotext += _('\nPlease be patient - this operation can take some time!')
                popupwin = encfsmanpopup.PopupWin(self, infotext)
                pos = self.ClientToScreen((0,0))  # pos is the left top edge of the win
                sz = self.GetSize()               # sz is the size of the win
                popupwin.Position((pos[0] + sz[0] / 2 - 150, pos[1] - 50 - sz[1] / 2), (0, sz[1]))
                popupwin.Popup()
                wx.SafeYield(self)            
            cmdline = '"%s" -S "%s" "%s"' % (encfsbinarypath, mapping.encdir, mapping.unencdir)
            proc = subprocess.Popen(cmdline, shell = True, stdin = subprocess.PIPE, stdout = subprocess.PIPE, stderr = subprocess.PIPE)
            wx.SafeYield(self)
            stdoutdata, stderrdata = proc.communicate(input = 'p\n' + mapping.passphrase)
        if withdialog: popupwin.Destroy()        
        if len(stderrdata) + len(stdoutdata) \
           and 'chaining' not in stderrdata and 'chaining' not in stdoutdata:
            self.writeToLogwin(_("Un-/Mount - Mount problem"))
            if withdialog:
                if len(stderrdata):
                    dlg = wx.MessageDialog(self, _('EncFS binary reports:\n\n%s') % (stderrdata), TITLE + ' - Mount problem', wx.OK | wx.ICON_WARNING)
                else:
                    dlg = wx.MessageDialog(self, _('EncFS binary reports:\n\n%s') % (stdoutdata), TITLE + ' - Mount problem', wx.OK | wx.ICON_WARNING)                    
                dlg.ShowModal()
            return False
        else:
            self.writeToLogwin(_("Un-/Mount - '%s' mounted to '%s'") % (mapping.encdir, mapping.unencdir)) 
            self.taskbarNotify(_('mapping mounted successfully'))
   
        wx.SafeYield(self)
        return True
    #-------------------------------------------------------------------------------------------

    def onAutomount(self, event):
        allmounted = True
        minfo = self.getMountInfos()
        for mapping in self.mappings:
            if mapping.automount == 'True' and mapping.unencdir not in minfo:
                if not self.createAndMount(mapping):
                    allmounted = False
        if not allmounted and self.retrycount > 0:
            self.automounttimer.Start(5000, oneShot = True)
            self.retrycount -= 1
        self.updateListwin()
    #-------------------------------------------------------------------------------------------
                      
    def requestPassphrase(self, mapping):
        dlg = encfsmanpasswd.PassphraseDialog(self, self.config, mapping)
        return dlg.ShowModal() == wx.ID_OK
    #-------------------------------------------------------------------------------------------

    def onItemDoubleClick(self, event=None, mount = None):
        index, mapping = self.getSelectedMapping()
        minfo = self.getMountInfos()
        if index < 0:
            self.writeToLogwin(_("Un-/Mount - nothing to do - no mapping selected"))
        else:
            if (mount == True) or (mapping.unencdir not in minfo and mount == None): # mount
                if not self.keyringisopen and not self.requestPassphrase(mapping): return
                if len(mapping.passphrase) == 0:
                    if not self.requestPassphrase(mapping): return
                self.createAndMount(mapping, True)                
            elif (mount == False) or (mapping.unencdir in minfo and mount == None): # unmount
                if os.path.isfile(self.config['Misc']['fusebin']):
                    cmdline = '"%s" -u "%s"' % (self.config['Misc']['fusebin'], mapping.unencdir)
                    proc = subprocess.Popen(cmdline, shell = True, stdin = subprocess.PIPE, stdout = subprocess.PIPE, stderr = subprocess.PIPE)
                    stdoutdata, stderrdata = proc.communicate()
                    if len(stderrdata):
                        self.writeToLogwin(_("Un-/Mount - Unmount problem"))
                        dlg = wx.MessageDialog(self, _('fusermount binary reports:\n\n%s') % (stderrdata), TITLE + ' - Unmount problem', wx.OK | wx.ICON_WARNING)
                        dlg.ShowModal()
                    else:
                        self.writeToLogwin(_("Un-/Mount - '%s' unmounted") % (mapping.unencdir))
                        self.taskbarNotify(_('mapping unmounted successfully'))
            self.updateListwin()
    #-------------------------------------------------------------------------------------------

    def onMount(self, event):
        self.onItemDoubleClick(mount = True)
    #-------------------------------------------------------------------------------------------

    def onUnmount(self, event):
        self.onItemDoubleClick(mount = False)
    #-------------------------------------------------------------------------------------------
        
    def getSelectedMapping(self):
        if self.listwin.GetSelectedItemCount():
            state = wx.LIST_STATE_SELECTED            
            index = self.listwin.GetNextItem(-1, wx.LIST_NEXT_ALL, wx.LIST_STATE_SELECTED)
            mapping = self.mappings[index]
            return index, mapping
        else:
            return -1, None
    #-------------------------------------------------------------------------------------------
    
    def onEditMapping(self, event):
        index, mapping = self.getSelectedMapping()
         
        if index < 0:
            self.writeToLogwin(_("Edit mapping - no mapping selected"))        
        elif self.listwin.GetItem(index, 3).GetText() == 'True':
            self.writeToLogwin(_("Edit mapping - can't edit mounted mapping"))
            dlg = wx.MessageDialog(self, _('This mapping is mounted - unmount first'), TITLE + _(' - Edit mapping'), wx.OK | wx.ICON_INFORMATION)
            dlg.ShowModal()
        elif not self.keyringisopen:
            self.writeToLogwin("Edit mapping - no access to keyring")
            dlg = wx.MessageDialog(self, _('Keyring / KDEWallet not accessible / open'), TITLE + _(' - Edit mapping'), wx.OK | wx.ICON_INFORMATION)
            dlg.ShowModal()
            self.checkKeyring()
        else:
            dlg = encfsmanedit.EditDialog(self, self.writeToLogwin, self.config, mapping, True)
            if dlg.ShowModal() == wx.ID_OK:
                self.mappings[index] = mapping
                self.writeToLogwin(_("Edit mapping - '%{0}s, %{1}s, %{2}s' modified".format(mapping.unencdir, mapping.encdir, mapping.automount)))
                keyring.set_password(mapping.idstring, USER, mapping.passphrase)
                self.handleConfigData('w')
            else:
                self.handleConfigData('r')
            self.updateListwin()
    #-------------------------------------------------------------------------------------------
    
    def onAddMapping(self, event):
        ids = '%s-%s-%s' % (TITLE, strftime('%Y%m%d%H%M%S'), idGenerator()) # create a unique mapping identifier
        mapping = Mapping(idstring = ids)
        dlg = encfsmanedit.EditDialog(self, self.writeToLogwin, self.config, mapping, False)
        if dlg.ShowModal() == wx.ID_OK:
            self.mappings.append(mapping)
            self.writeToLogwin(_("Add mapping - '%{0}s, %{1}s, %{2}s' added to list".format(mapping.unencdir, mapping.encdir, mapping.automount)))
            if self.keyringisopen:
                keyring.set_password(mapping.idstring, USER, mapping.passphrase)
            else:
                self.writeToLogwin(_("Add mapping - no access to keyring - passphrase can not be stored"))
                dlg = wx.MessageDialog(self, _('Keyring / KDEWallet not accessible / open'), TITLE + _(' - Add mapping'), wx.OK | wx.ICON_INFORMATION)
                dlg.ShowModal()
                self.checkKeyring(fetch = False)
            if mapping.autocreate == 'True':
                self.onCreate(mapping = mapping)
            self.updateListwin()
            self.handleConfigData('s')
    #-------------------------------------------------------------------------------------------

    def onDelMapping(self, event):
        index, mapping = self.getSelectedMapping()
        if index >= 0 and self.listwin.GetItem(index, 3).GetText() == 'False':
            del self.mappings[index]
            self.writeToLogwin(_("Delete mapping - '%{0}s, %{1}s, %{2}s' deleted from list".format(mapping.unencdir, mapping.encdir, mapping.automount)))
            self.updateListwin()
            self.handleConfigData('s')
        elif index < 0:
            self.writeToLogwin(_('Delete mapping - no mapping selected'))
        else:
            self.writeToLogwin(_("Delete mapping - can't delete mounted mapping"))
            dlg = wx.MessageDialog(self, _('This mapping is mounted - unmount first'), TITLE + _(' - Delete mapping'), wx.OK | wx.ICON_INFORMATION)
            dlg.ShowModal()           
    #-------------------------------------------------------------------------------------------

    def onPreferences(self, event):
        """  """
        self.handleConfigData('s')
        prefdlg = encfsmanprefs.PrefsDialog(self, self.hlcolor, self.inifilename)
        if prefdlg.ShowModal() == wx.ID_OK:
            self.config = prefdlg.getNewConfig()
            self.handleConfigData('s')
            self.writeToLogwin(_('Preferences - new values set'))
        else:
            self.handleConfigData('r')
        prefdlg.Destroy()
        self.myfont = wx.Font(int(self.config['Gui']['fsize']),
                              wx.DEFAULT, wx.NORMAL, wx.NORMAL, False,
                              self.config['Gui']['font'])
        self.listwin.SetFont(self.myfont)
        collist = self.config['Gui']['hlcolor']
        self.hlcolor = (int(collist[0]), int(collist[1]), int(collist[2]))
        self.updateListwin()
        self.retrycount = int(self.config['Misc']['retry'])
        self.automounttimer.Start(3000, oneShot = True)
        self._mgr.Update()
        self.Unbind(wx.EVT_CLOSE)
        if self.config['Gui']['systray'] == 'True':
            self.Bind(wx.EVT_CLOSE, self.onSysTray)
        else:
            self.Bind(wx.EVT_CLOSE, self.onCloseWindow)            
    #-------------------------------------------------------------------------------------------

    def onProperties(self, event):
        if self.listwin.GetSelectedItemCount():
            index = -1
            state = wx.LIST_STATE_SELECTED            
            index = self.listwin.GetNextItem(index, wx.LIST_NEXT_ALL, state)
            id_title = self.listwin.getColumnText(index, 0)
            if not isSpecialId(id_title):
                win = mumapopup.PropertyPopup(self, self.config, self.mumadb, id_title)
                pos = self.listwin.ClientToScreen((0,0))  # pos is the left top edge of the win
                sz = self.listwin.GetSize()               # sz is the size of the win
                win.Position((pos[0] + sz[0] / 2, pos[1] - sz[1] / 2), (0, sz[1]))       
                win.Popup()
    #-------------------------------------------------------------------------------------------

    def onAbout(self, event):
        dlg = encfsmanabout.AboutDialog(self)
        dlg.ShowModal()
    #-------------------------------------------------------------------------------------------

    def setWindowDefaults(self):
        self.config['Window']['posx'] = 0
        self.config['Window']['posy'] = 0
        self.config['Window']['sizex'] = 500
        self.config['Window']['sizey'] = 270
        self.config['Window']['lwcol0'] = 180
        self.config['Window']['lwcol1'] = 180
        self.config['Window']['lwcol2'] = 80
        self.config['Window']['lwcol3'] = 60
        self.config['Window']['dirposx'] = 0
        self.config['Window']['dirposy'] = 0
        self.config['Window']['dirsizex'] = 390
        self.config['Window']['dirsizey'] = 450                        
    #-------------------------------------------------------------------------------------------
        
    def handleConfigData(self, rws = 'r'):
        if rws == 'r':                              # read  / check ini file
            inipath = HOMEDIR
            if len(sys.argv) > 1:
                argument = sys.argv[1].lower()
                if argument.startswith('-i='):
                    inipath = sys.argv[1][3:]
            self.inifilename = os.path.join(inipath, TITLE + '.ini')
            self.config = ConfigObj(self.inifilename, encoding = 'UTF8')
            if 'Version' not in self.config:              # new, empty ini file
                # initialize all values
                self.config.initial_comment = ['EncFSman preferences file']
                self.config['Version'] = {}
                self.config['Version']['vernum'] = VERNUM
                self.config['Version']['startup'] = ''
                self.config['Version']['shutdown'] = ''
                self.config['Window'] = {}
                self.setWindowDefaults()
                self.config['Window']['perspective'] = ''
                self.config['Window']['defaultperspective'] = ''                
                self.config['View'] = {}                
                self.config['View']['logwin'] = 'True'
                self.config['Gui'] = {}
                self.config['Gui']['grid'] = 'True'
                self.config['Gui']['font'] = wx.SystemSettings_GetFont(wx.SYS_ANSI_VAR_FONT).GetFaceName()
                self.config['Gui']['fsize'] = '9'
                self.config['Gui']['hlcolor'] = (0, 160, 20)
                self.config['Gui']['startmin'] = 'False'
                self.config['Gui']['norm'] = 'True'
                self.config['Gui']['notify'] = 'True'
                self.config['Gui']['systray'] = 'True'
                self.config['Gui']['language'] = ('Default', 'English', 'German')
                self.config['Gui']['curlang'] = 'Default'
                self.config['Encfs'] = {}
                self.config['Encfs']['mappings'] = []
                self.config['Misc'] = {}
                self.config['Misc']['actionlog'] = 'True'
                self.config['Misc']['update'] = 'False'
                self.config['Misc']['instance'] = 'False'   
                self.config['Misc']['encfsbin'] = 'encfs'
                self.config['Misc']['encfsctl'] = 'encfsctl'
                self.config['Misc']['fusebin'] = 'fusermount'
                self.config['Misc']['retry'] = '3'
                self.config['Misc']['hiddendirs'] = 'True'
                self.config['Version']['vernum'] = VERNUM
                self.config.write()
            elif 'language' not in self.config['Gui']:              # added in 1.03  (Python 3)
                self.config['Gui']['language'] = ('Default', 'English', 'German')
                self.config['Gui']['curlang'] = 'Default'                
                if 'dirposx' not in self.config['Window']:          # added in 0.60  (Python 3)              
                    self.config['Window']['dirposx'] = 0
                    self.config['Window']['dirposy'] = 0
                    self.config['Window']['dirsizex'] = 390
                    self.config['Window']['dirsizey'] = 450
                    self.config['Misc']['hiddendirs'] = 'True'
                if 'systray' not in self.config['Gui']:             # added in 0.20
                    self.config['Gui']['systray'] = 'True'
                self.writeToLogwin('Config - ini file (re)loaded')
            
            self.mappings = []
            for item in  self.config['Encfs']['mappings']:
                self.mappings.append(Mapping(inittuple = item))
            self.checkKeyring()
            
            self.config['Version']['startup'] = strftime('%Y-%m-%d %H:%M:%S') 
        else:          # write ini file rws = w or rws = s
            self.config['Version']['vernum'] = VERNUM
            self.config['Window']['perspective'] = self._mgr.SavePerspective()
            windowrect = self.GetRect()
            self.config['Window']['posx'] = windowrect[0]
            self.config['Window']['posy'] = windowrect[1]
            self.config['Window']['sizex'] = windowrect[2]
            self.config['Window']['sizey'] = windowrect[3]
            self.config['Encfs']['mappings'] = []
            for mapping in self.mappings:
                self.config['Encfs']['mappings'].append(mapping.getTuple())
            if rws == 'w':
                self.config['Version']['shutdown'] = strftime('%Y-%m-%d %H:%M:%S')                        
            self.config.write()
    #-------------------------------------------------------------------------------------------

    def readUrl(self):    
        if sys.version < '3':
            import urllib2
        else:
            import urllib.request, urllib.error, urllib.parse

        try:
            if sys.version < '3':
                f = urllib2.urlopen(URL_DOWNLOAD_LIST, timeout = 5)
            else:
                f = urllib.request.urlopen(URL_DOWNLOAD_LIST, timeout = 5)
            html = f.read()
            f.close()
        except:
            return            
        
        versions = []
        while len(html):
            fpos = html.find('EncFSman-')
            html = html[fpos:]
            version = html[9:13]
            if html[13:17] == '.zip':
                versions.append(version)
            html = html[24:]
            
        versions.sort(reverse = True)     
        self.versions = versions
    #-------------------------------------------------------------------------------------------

    def onUpdate(self, event):
        # check for new program release
        self.versions = -1
        self.writeToLogwin(_('Update - try to connect to EncFSman website'))

        updatethread = threading.Thread(target = self.readUrl, args = ())
        updatethread.start()
        while updatethread.isAlive():
            try:
                wx.SafeYield(self)
            except:
                pass
        
        if type(self.versions) == int:
            dlg = wx.MessageDialog(self, _('unable to connect to the EncFSman website!'), TITLE + _(' - find EncFSman update'), wx.OK | wx.ICON_ERROR)
            dlg.ShowModal()
            return            
        
        self.writeToLogwin(_('Update - successfully connected to EncFSman website'))  
        if len(self.versions):
            version = self.versions[0]
            if version > VERNUM[:4]:
                dlg = encfsmanupdate.UpdateDialog(self, _("a newer version (%s) of EncFSman is available!") % (version))
                doit = dlg.ShowModal()
                self.mainmenu.Enable(self.ID_INSTALL, True)
                self.newrelease = '%s/%s-%s.zip' % (URL_DOWNLOAD_FILES, TITLE, version)
                if doit == wx.ID_OK:
                    self.onInstallUpdate()
                return
        else:
            self.writeToLogwin(_("Update check - cannot read information from EncFSman website"))
            return
            
        if self.updatecheck:
            self.updatecheck = False
        else:    
            dlg = wx.MessageDialog(self, _('the installed version of EncFSman is up to date!'), TITLE + _(' - find update'), wx.OK | wx.ICON_INFORMATION)
            dlg.ShowModal()
        self.writeToLogwin(_("Update check - the installed version of EncFSman is up to date"))
    #-------------------------------------------------------------------------------------------

    def onInstallUpdate(self, event = None):
        # install new program release
        dlg = encfsmaninstall.InstallDialog(self, self.config, self.logger.info, self.newrelease)
        if dlg.ShowModal() == wx.ID_OK:
            self.writeToLogwin(_('Update - new release installed - restart %s') % TITLE)
            self.mainmenu.Enable(self.ID_INSTALL, False)    
    #-------------------------------------------------------------------------------------------

    def onWebsite(self, event):
        try:
            webbrowser.open(URL_WEBSITE)
        except:
            dlg = wx.MessageDialog(self, _('cannot connect to the EncFSman website!'), TITLE + _(' - browse website'), wx.OK | wx.ICON_ERROR)
            dlg.ShowModal()
    #-------------------------------------------------------------------------------------------

    def onHandbook(self, event):
        """ show Html help """
        webbrowser.open(os.path.join(APPPATH, 'encfsman.html'))
    #-------------------------------------------------------------------------------------------

    def onSysTray(self, event = None):
        if event != None:
            self.handleConfigData('w')          # save window position
        self.tbicon.minimizeToTaskbar()
    #-------------------------------------------------------------------------------------------

    def OnRestoreWindow(self, event):
        self.Show(True)
        self.Iconize(False)
        self.tbicon.RemoveIcon()
        self.Raise()       
    #-------------------------------------------------------------------------------------------------------
    
    def onCloseWindow(self, event = None):
        self.handleConfigData('w')
        self.logger.info(_('EncFSman shutdown\n'))
        logging.shutdown()
        evtmgr.eventManager.DeregisterListener(self.onContext)
        self._mgr.UnInit()          # deinitialize the frame manager
        self.Destroy()              # switch the light off
# - class EncFSmanFrame(wx.Frame):------------------------------------------------------------------


class EncFSmanApp(wx.App):
    def OnInit(self):
        inipath = HOMEDIR
        if len(sys.argv) > 1:
            argument = sys.argv[1].lower()
            if argument.startswith('-i='):
                inipath = sys.argv[1][3:]
        inifilename = os.path.join(inipath, TITLE + '.ini')
        config = ConfigObj(inifilename, encoding = 'UTF8')
        
        try:
            si = SingleInstance(TITLE)
        except:
            return True
                   
        frame = EncFSmanFrame(None, -1, VERSION, si, keyringisthere)
        frame.Show()
        return True
    #-------------------------------------------------------------------------------------------

    def OnExit(self):
        try:
            self.lock.release()
        except:
            pass                    # we did our best
# - class EncFSmanApp(wx.App):------------------------------------------------------------------


    
