#!/usr/bin/env python
#-*- coding: UTF-8 -*-
 
import os.path
import wx
from . import encfsmandir
from   .encfsmanmisc import *


class PrefsDialog(wx.Dialog):
    """ all settings for this app """
    ID_BTN_COLOUR = wx.NewId()
    ID_RADIOBOX = wx.NewId()
    ID_RADIOBUTTON1 = wx.NewId()
    ID_RADIOBUTTON2 = wx.NewId()
    ID_CB_GRID = wx.NewId()
    ID_CB_INSTANCE = wx.NewId()
    ID_CB_NOTIFY = wx.NewId()
    ID_CB_STAY = wx.NewId()
    ID_CB_MULTI = wx.NewId()
    ID_CB_ADDWARN = wx.NewId()
    ID_CB_UPDATE = wx.NewId()
    ID_CB_HIDDEN = wx.NewId()
    ID_CB_ERRORLOG = wx.NewId()
    ID_CB_ACTIONLOG = wx.NewId()
    ID_CB_NORM = wx.NewId()
    ID_CB_MINI = wx.NewId()
    ID_BTN_COLOUR = wx.NewId()
    ID_BTN_ENCFSBIN = wx.NewId()
    ID_BTN_FUSEBIN = wx.NewId()
    ID_BTN_ENCFSCTL = wx.NewId()
    
    
    def __init__(self, parent, hlcolor, inifilename):
        """ place preferences items on a dialog with 4 tabs """
        wx.Dialog.__init__(self, parent, -1, TITLE + _(' - Preferences'))
        self.config = parent.config
        self.hlcolor = hlcolor
        self.readonly_color = wx.SystemSettings_GetColour(READONLY_COLOR)
        self.Centre()
        btnadj = int(self.config['Gui']['norm'] == 'False')

        self.sizer = wx.BoxSizer(wx.VERTICAL)
        row1 = wx.BoxSizer(wx.HORIZONTAL)
        self.nb = wx.Notebook(self, -1)
        row1.Add(self.nb) #, 1, wx.GROW | wx.ALL, GAP)        
        self.initTab1()
        self.initTab2(inifilename)
        
        self.butok1 = wx.Button(self, wx.ID_OK, '&OK')
        self.Bind(wx.EVT_BUTTON, self.onOk, id = wx.ID_OK)
        self.butcancel1 = wx.Button(self, wx.ID_CANCEL, '&Cancel')
        self.butok2 = wx.Button(self, wx.ID_OK, '&OK')
        self.Bind(wx.EVT_BUTTON, self.onOk, id = wx.ID_OK)
        self.butcancel2 = wx.Button(self, wx.ID_CANCEL, '&Cancel')
         
        row21 = wx.BoxSizer(wx.HORIZONTAL)
        row21.AddStretchSpacer()
        row21.Add(self.butok1, 0, wx.GROW | wx.ALL, GAP)
        row21.Add(self.butcancel1, 0, wx.GROW | wx.ALL, GAP)
        row22 = wx.BoxSizer(wx.HORIZONTAL)
        row22.Add(self.butok2, 1, wx.GROW | wx.ALL, GAP)
        row22.Add(self.butcancel2, 1, wx.GROW | wx.ALL, GAP)
        self.onCheckNorm()                    
        self.sizer.Add(row1, 1, wx.GROW | LRT, BORDER)
        self.sizer.Add(wx.BoxSizer(wx.HORIZONTAL), 0, wx.GROW | LRT, GAP)
        self.sizer.Add(row21, 0, wx.GROW | wx.LEFT | wx.RIGHT, GAP)
        self.sizer.Add(row22, 0, wx.GROW | wx.LEFT | wx.RIGHT , GAP)
        self.sizer.Add(wx.BoxSizer(wx.HORIZONTAL), 0, wx.GROW | wx.BOTTOM, GAP)
        self.SetSizer(self.sizer)
        self.Fit()
        self.Center()
    #-------------------------------------------------------------------------------------------

    def initTab1(self):
        page1 = wx.Panel(self.nb, -1)
        p1sizer = wx.BoxSizer(wx.VERTICAL)  
        lcsz = wx.FlexGridSizer(9, 3, 0, 0)
        lcsz.AddGrowableCol(1)
        st = wx.StaticText(page1, -1, _('with Grid:     '))
        lcsz.Add(st, 0, wx.ALL, GAP)
        self.cbg = wx.CheckBox(page1, self.ID_CB_GRID, _(' (change applied after program restart)')) 
        lcsz.Add(self.cbg, 0, wx.ALL, GAP)
        self.Bind(wx.EVT_CHECKBOX, self.onCheckGrid, id = self.ID_CB_GRID)
        self.cbg.SetValue(self.config['Gui']['grid'] == 'True')
        lcsz.AddStretchSpacer()                
        enu = wx.FontEnumerator()
        enu.EnumerateFacenames()
        fontlist = enu.GetFacenames()
        fontlist.sort()        
        st = wx.StaticText(page1, -1, _('Font family:    '))
        lcsz.Add(st, 0, wx.ALL, GAP)
        cbfont = wx.ComboBox(page1, -1, self.config['Gui']['font'], 
                             choices = fontlist, style = wx.CB_DROPDOWN | wx.CB_READONLY)
        lcsz.Add(cbfont, 0, wx.GROW | wx.ALL, GAP)
        self.Bind(wx.EVT_COMBOBOX, self.onFontComboBox, cbfont)
        try:
            cbfont.SetSelection(fontlist.index(self.config['Gui']['font']))
        except:
            pass
        lcsz.AddStretchSpacer()
        
        st = wx.StaticText(page1, -1, _('Font size:'))
        lcsz.Add(st, 0, wx.ALL, GAP)
        self.scf = wx.SpinCtrl(page1, -1)
        self.scf.SetRange(6, 16)           
        self.scf.SetValue(int(self.config['Gui']['fsize']))
        self.Bind(wx.EVT_SPINCTRL, self.onFontSize, self.scf)
        lcsz.Add(self.scf, 0, wx.GROW | wx.ALL, GAP)
        lcsz.AddStretchSpacer()

        st = wx.StaticText(page1, -1, _('Language:'))
        lcsz.Add(st, 0, wx.ALL, GAP)
        cblang = wx.ComboBox(page1, -1, self.config['Gui']['curlang'], 
                             choices = self.config['Gui']['language'], style = wx.CB_DROPDOWN | wx.CB_READONLY)
        lcsz.Add(cblang, 0, wx.GROW | wx.ALL, GAP)
        self.Bind(wx.EVT_COMBOBOX, self.onLanguageComboBox, cblang)
        lcsz.AddStretchSpacer()

        lcsz.AddStretchSpacer()
        st = wx.StaticText(page1, -1, (_('Language change applied after program restart')))
        font = wx.Font(8, wx.DEFAULT, wx.NORMAL, wx.NORMAL)
        st.SetFont(font)
        lcsz.Add(st, 0, wx.ALL, GAP)
        lcsz.AddStretchSpacer()

        st = wx.StaticText(page1, -1, _('Highlight colour:'))
        lcsz.Add(st, 0, wx.ALL, GAP)
        self.tccolor = wx.TextCtrl(page1, -1, ' %s ' % (str(self.hlcolor)), style = wx.TE_READONLY)
        self.tccolor.SetBackgroundColour(self.hlcolor)
        lcsz.Add(self.tccolor, 1, wx.ALL | wx.GROW, GAP)
        butcolour = wx.Button(page1, self.ID_BTN_COLOUR, _(' New colour '))
        self.Bind(wx.EVT_BUTTON, self.onColour, id = self.ID_BTN_COLOUR)
        lcsz.Add(butcolour, 1, wx.ALL | wx.GROW, GAP)

        st = wx.StaticText(page1, -1, _('Start minimized:'))
        lcsz.Add(st, 0, wx.ALL, GAP)
        self.cbm = wx.CheckBox(page1, self.ID_CB_MINI, _(" (if checked - start app in systray)"))
        lcsz.Add(self.cbm, 0, wx.ALL, GAP)
        self.Bind(wx.EVT_CHECKBOX, self.onCheckMini, id = self.ID_CB_MINI)
        self.cbm.SetValue(self.config['Gui']['startmin'] == 'True')
        lcsz.AddStretchSpacer()

        st = wx.StaticText(page1, -1, _('Button align:'))
        lcsz.Add(st, 0, wx.ALL, GAP)
        self.cby = wx.CheckBox(page1, self.ID_CB_NORM, _(" (if checked - 'norm' style - right aligned)"))
        lcsz.Add(self.cby, 0, wx.ALL, GAP)
        self.Bind(wx.EVT_CHECKBOX, self.onCheckNorm, id = self.ID_CB_NORM)
        self.cby.SetValue(self.config['Gui']['norm'] == 'True')
        lcsz.AddStretchSpacer()

        st = wx.StaticText(page1, -1, _('Notification:'))
        lcsz.Add(st, 0, wx.ALL, GAP)
        self.cbn = wx.CheckBox(page1, self.ID_CB_NOTIFY, _(' (if checked - enable systray notification)'))
        lcsz.Add(self.cbn, 0, wx.ALL, GAP)
        self.Bind(wx.EVT_CHECKBOX, self.onCheckNotify, id = self.ID_CB_NOTIFY)
        self.cbn.SetValue(self.config['Gui']['notify'] == 'True')
        lcsz.AddStretchSpacer()

        st = wx.StaticText(page1, -1, _('Stay in systray:'))
        lcsz.Add(st, 0, wx.ALL, GAP)
        self.cbs = wx.CheckBox(page1, self.ID_CB_STAY, _(' (if checked - stay in systray on close)'))
        lcsz.Add(self.cbs, 0, wx.ALL, GAP)
        self.Bind(wx.EVT_CHECKBOX, self.onCheckStay, id = self.ID_CB_STAY)
        self.cbs.SetValue(self.config['Gui']['systray'] == 'True')
        lcsz.AddStretchSpacer()

        p1sizer.Add(lcsz, 0, wx.GROW | LRT, BORDER)        
        page1.SetSizer(p1sizer)
        self.nb.AddPage(page1, _(' Gui '))
    #-------------------------------------------------------------------------------------------

    def initTab2(self, inifilename):
        page2 = wx.Panel(self.nb, -1)
        p2sizer = wx.BoxSizer(wx.VERTICAL)  
        miscsz = wx.FlexGridSizer(8, 3, 0, 0)
        miscsz.AddGrowableCol(1)
        
        st = wx.StaticText(page2, -1, TITLE + _(' Ini path:'))
        miscsz.Add(st, 0, wx.ALL, GAP)
        self.ctrlini = wx.TextCtrl(page2, -1, inifilename, style = wx.TE_READONLY)
        self.ctrlini.SetBackgroundColour(self.readonly_color)
        miscsz.Add(self.ctrlini, 0, wx.ALL | wx.GROW, GAP)
        miscsz.AddStretchSpacer()

        st = wx.StaticText(page2, -1, _('Encfs binary path:'))
        miscsz.Add(st, 0, wx.ALL, GAP)
        self.ctrlenc = wx.TextCtrl(page2, -1, self.config['Misc']['encfsbin'], style = wx.TE_READONLY)
        self.ctrlenc.SetBackgroundColour(self.readonly_color)
        miscsz.Add(self.ctrlenc, 0, wx.ALL | wx.GROW, GAP)
        self.btnencfsb = wx.Button(page2, self.ID_BTN_ENCFSBIN, _('Browse')) 
        self.Bind(wx.EVT_BUTTON, self.onEncfsBin, id = self.ID_BTN_ENCFSBIN)
        miscsz.Add(self.btnencfsb, 0, wx.ALL, GAP)

        st = wx.StaticText(page2, -1, _('Encfsctl binary path:'))
        miscsz.Add(st, 0, wx.ALL, GAP)
        self.ctrlctl = wx.TextCtrl(page2, -1, self.config['Misc']['encfsctl'], style = wx.TE_READONLY)
        self.ctrlctl.SetBackgroundColour(self.readonly_color)
        miscsz.Add(self.ctrlctl, 0, wx.ALL | wx.GROW, GAP)
        self.btnencctl = wx.Button(page2, self.ID_BTN_ENCFSCTL, _('Browse')) 
        self.Bind(wx.EVT_BUTTON, self.onEncfsCtl, id = self.ID_BTN_ENCFSCTL)
        miscsz.Add(self.btnencctl, 0, wx.ALL, GAP)

        st = wx.StaticText(page2, -1, _('Fusermount path:'))
        miscsz.Add(st, 0, wx.ALL, GAP)
        self.ctrlfuse = wx.TextCtrl(page2, -1, self.config['Misc']['fusebin'], style = wx.TE_READONLY)
        self.ctrlfuse.SetBackgroundColour(self.readonly_color)
        miscsz.Add(self.ctrlfuse, 0, wx.ALL | wx.GROW, GAP)
        self.btnfuseb = wx.Button(page2, self.ID_BTN_FUSEBIN, _('Browse')) 
        self.Bind(wx.EVT_BUTTON, self.onFuseBin, id = self.ID_BTN_FUSEBIN)
        miscsz.Add(self.btnfuseb, 0, wx.ALL, GAP)

        st = wx.StaticText(page2, -1, _('Hidden dirs:'))
        miscsz.Add(st, 0, wx.ALL, GAP)
        self.cbd = wx.CheckBox(page2, self.ID_CB_HIDDEN, _(' (if checked - show hidden dirs / files)'))
        miscsz.Add(self.cbd, 0, wx.ALL, GAP)
        self.Bind(wx.EVT_CHECKBOX, self.onCheckHidden, id = self.ID_CB_HIDDEN)
        self.cbd.SetValue(self.config['Misc']['hiddendirs'] == 'True')
        miscsz.AddStretchSpacer()
                
        st = wx.StaticText(page2, -1, _('Retry count:'))
        miscsz.Add(st, 0, wx.ALL, GAP)
        self.scr = wx.SpinCtrl(page2, -1)
        self.scr.SetRange(0, 10)
        self.scr.SetValue(int(self.config['Misc']['retry']))
        self.Bind(wx.EVT_SPINCTRL, self.onRetry, self.scr)
        miscsz.Add(self.scr, 0, wx.GROW | wx.ALL, GAP)
        miscsz.AddStretchSpacer()
        
        st = wx.StaticText(page2, -1, _('Multiple Instance:'))
        miscsz.Add(st, 0, wx.ALL, GAP)
        self.cbi = wx.CheckBox(page2, self.ID_CB_INSTANCE, _(' (change applied after program restart)'))
        miscsz.Add(self.cbi, 0, wx.ALL, GAP)
        self.Bind(wx.EVT_CHECKBOX, self.onCheckInstance, id = self.ID_CB_INSTANCE)
        self.cbi.SetValue(self.config['Misc']['instance'] == 'True')
        miscsz.AddStretchSpacer()

        st = wx.StaticText(page2, -1, _('Update check:'))
        miscsz.Add(st, 0, wx.ALL, GAP)
        self.cbu = wx.CheckBox(page2, self.ID_CB_UPDATE, _(' (if checked - auto examine website)'))
        miscsz.Add(self.cbu, 0, wx.ALL, GAP)
        self.Bind(wx.EVT_CHECKBOX, self.onCheckUpdate, id = self.ID_CB_UPDATE)
        self.cbu.SetValue(self.config['Misc']['update'] == 'True')
        miscsz.AddStretchSpacer()

        st = wx.StaticText(page2, -1, _('Action log:'))
        miscsz.Add(st, 0, wx.ALL, GAP)
        self.cba = wx.CheckBox(page2, self.ID_CB_ACTIONLOG, _(' (if checked - write Log messages)'))
        miscsz.Add(self.cba, 0, wx.ALL, GAP)
        self.Bind(wx.EVT_CHECKBOX, self.onCheckActionlog, id = self.ID_CB_ACTIONLOG)
        self.cba.SetValue(self.config['Misc']['actionlog'] == 'True')
        miscsz.AddStretchSpacer()

        p2sizer.Add(miscsz, 0, wx.GROW | LRT, BORDER)        
        page2.SetSizer(p2sizer)
        self.nb.AddPage(page2, _(' Misc '))
    #-------------------------------------------------------------------------------------------

    def onButtonAdd(self, event):
        self.tcft.SetEditable(False)
        self.msgtext.SetLabel(self.NOEDITMSG)
        font = wx.Font(8, wx.DEFAULT, wx.NORMAL, wx.NORMAL)
        self.msgtext.SetFont(font)
        self.tcft.Clear()
        self.isadd = True
        self.onEdit()
    #-------------------------------------------------------------------------------------------
    
    def onSelected(self, event):
        selectindex = event.GetSelection()
        if selectindex >= 0:
            self.selectindex = selectindex
            self.tcft.Clear()
            self.tcft.SetEditable(False)
            self.tcft.SetBackgroundColour(wx.SystemSettings_GetColour(wx.SYS_COLOUR_BACKGROUND))
            self.msgtext.SetLabel(self.NOEDITMSG)
            font = wx.Font(8, wx.DEFAULT, wx.NORMAL, wx.NORMAL)
            self.msgtext.SetFont(font)
    #-------------------------------------------------------------------------------------------
    
    def onFontComboBox(self, event):
        self.config['Gui']['font'] = event.GetString()
    #-------------------------------------------------------------------------------------------

    def onLanguageComboBox(self, event):
        self.config['Gui']['curlang'] = event.GetString()
    #-------------------------------------------------------------------------------------------

    def onFontSize(self, event):
        self.config['Gui']['fsize'] = str(event.GetInt())
    #-------------------------------------------------------------------------------------------

    def onColour(self, event):
        data = wx.ColourData()
        data.SetColour(self.hlcolor)
        dlg = wx.ColourDialog(self, data)
        if dlg.ShowModal() == wx.ID_OK:
            data = dlg.GetColourData()
            self.hlcolor = data.GetColour().Get()
            self.tccolor.SetValue(' %s ' % (str(self.hlcolor)))
            self.tccolor.SetBackgroundColour(self.hlcolor)
    #-------------------------------------------------------------------------------------------

    def onCheckNorm(self, event = None):
        self.config['Gui']['norm'] = str(self.cby.GetValue())
        btnadj = int(self.config['Gui']['norm'] == 'False')
        if btnadj:
            self.butok1.Hide()
            self.butcancel1.Hide()
            self.butok2.Show()
            self.butcancel2.Show()
        else:
            self.butok1.Show()
            self.butcancel1.Show()
            self.butok2.Hide()
            self.butcancel2.Hide()
        self.sizer.Layout()
        self.Refresh()
    #-------------------------------------------------------------------------------------------

    def onEncfsBin(self, event):
        dlg = encfsmandir.DirDialog(self, TITLE + ' - select encfs bin dir', self.config, 
                                    self.config['Misc']['encfsbin'], mode = 'openf', filters = WILDEXEC)       
        if dlg.ShowModal() == wx.ID_OPEN:
            binpath = os.path.normpath(dlg.GetFilePath())      
            self.config['Misc']['encfsbin'] = binpath
            self.ctrlenc.SetValue(binpath)
        dlg.Destroy()
    #-------------------------------------------------------------------------------------------

    def onEncfsCtl(self, event):
        dlg = encfsmandir.DirDialog(self, TITLE + ' - select encfsctl bin dir', self.config, 
                                    self.config['Misc']['encfsctl'], mode = 'openf', filters = WILDEXEC)             
        if dlg.ShowModal() == wx.ID_OPEN:
            binpath = os.path.normpath(dlg.GetPath())      
            self.config['Misc']['encfsctl'] = binpath
            self.ctrlctl.SetValue(binpath)
        dlg.Destroy()        
    #-------------------------------------------------------------------------------------------
       
    def onFuseBin(self, event):        
        dlg = encfsmandir.DirDialog(self, TITLE + ' - select fuse bin dir', self.config, 
                                    self.config['Misc']['fusebin'], mode = 'openf', filters = WILDEXEC)            
        if dlg.ShowModal() == wx.ID_OPEN:
            binpath = os.path.normpath(dlg.GetPath())      
            self.config['Misc']['fusebin'] = binpath
            self.ctrlfuse.SetValue(binpath)
        dlg.Destroy()
    #-------------------------------------------------------------------------------------------
        
    def onRetry(self, event):
        self.config['Misc']['retry'] = str(event.GetInt())
    #-------------------------------------------------------------------------------------------
    
    def onOk(self, event):
        event.Skip()
    #-------------------------------------------------------------------------------------------
    
    def onCheckHidden(self, event):
        self.config['Misc']['hiddendirs'] = str(self.cbd.GetValue())
    #-------------------------------------------------------------------------------------------

    def onCheckMini(self, event):
        self.config['Gui']['startmin'] = str(self.cbm.GetValue())
    #-------------------------------------------------------------------------------------------

    def onCheckGrid(self, event):
        """  """
        self.config['Gui']['grid'] = str(self.cbg.GetValue())
    #-------------------------------------------------------------------------------------------

    def onCheckInstance(self, event):
        self.config['Misc']['instance'] = str(self.cbi.GetValue())
    #-------------------------------------------------------------------------------------------

    def onCheckNotify(self, event):
        self.config['Gui']['notify'] = str(self.cbn.GetValue())
    #-------------------------------------------------------------------------------------------

    def onCheckStay(self, event):
        self.config['Gui']['systray'] = str(self.cbs.GetValue())
    #-------------------------------------------------------------------------------------------

    def onCheckUpdate(self, event):
        self.config['Misc']['update'] = str(self.cbu.GetValue())
    #-------------------------------------------------------------------------------------------

    def onCheckActionlog(self, event):
        """  """
        self.config['Misc']['actionlog'] = str(self.cba.GetValue())
    #-------------------------------------------------------------------------------------------

    def getNewConfig(self):
        """  """
        self.config['Gui']['hlcolor'] = self.hlcolor
        return self.config
# - class PrefDialog(wx.Dialog):----------------------------------------------------------------

